'-----------------------------------------------------------------------
' Program:  CONDEP -- LCM Conditional Dependence Diagnostics
'
' Purpose:  For a binary data latent class model, identifies
'           pairs of conditionally dependent manifest variables.
'
' Description
'
' For a Latent Class Model (LCM) of binary manifest variables (items),
' this program takes as input the observed and expected frequencies of
' each unique response pattern.  For each unique item pair, the
' program:
'
'    1.  Constrcts the two-way marginal tables for observed and
'        expected frequenices:,
'
'    2.  For the expected marginal table, calculates the log-odds
'        ratio and its standard error; and
'
'    3.  Calculates a z statistic to assess how much the log-odds
'        ratio of the corresponding observed marginal table deviates
'        from the expected value.  A p-value is calcuated for each z
'        statistic; this is used for comparison purposes only and
'        not viewed in a strict statistical way.
'
' For each item pair, deviance of the observed and expected
' two-way marginals is also expressed with the likelihood-ratio
' chi-squared (G-squared) statistic; again, this is used for
' comparison purposes only.
' ----------------------------------------------------------------------
' This program can be downloaded from the Latent Class Analysis
' web pages:
'
' http://ourworld.compuserve.com/homepages/jsuebersax/index.htm
'
' From there, navigate to the page that discusses Conditioal
' Dependence with LCMs.
'
' John Uebersax jsuebersx@yahoo.com
' May 2000
' ----------------------------------------------------------------------
' Technical Notes
'
' If a single cell in any 2-way marginal table is 0, .5 is to that
' cell and added or subtracted to the other cells to preserve the
' total.
'
' If a row or column marginal total in a two-way table is 0, the
' program will crash.  This should not occur, however, as it implies that
' an item is invariant--either always level 1 or always level 2.
' Still, this is not ideal programming and hopefully a future version
' will add a warning message for this.
' ----------------------------------------------------------------------
' Written:  05/30/00
' Author:   John Uebersax
' Revised   Change(s)
' -------   ------------------------------------------------------------
' ----------------------------------------------------------------------

      DIM IV(1024, 10)       '  Index vectors
      DIM FE(1024)           '  Expected frequences
      DIM FO(1024)           '  Observed frequences
      DIM marge(10, 10)        '  Expected 2-way marginal frequencies
      DIM margo(10, 10)        '  Observed 2-way marginal frequencies
      DIM psie(10, 10)         '  Expected log odds ratios
      DIM psio(10, 10)         '  Observed log odds ratios
      DIM sigma(10, 10)        '  std error for expected log odds ratios
      DIM g2(10, 10)           '  LR chi-squared
      MC = 2                 '  Number of manifest categories

      CLS

      INPUT "Number of items:  ", NR
'     INPUT "Number of response patterns:  ", NC
      nc = 2 ^ NR

      INPUT "Name of DATA file:  ", f1$
      INPUT "Name of OUTPUT file:  ", f2$
'      INPUT "Prefix for input/output filenames:  ", f$

      OPEN f1$ FOR INPUT AS #1
'     OPEN f$ + "technical.out" FOR OUTPUT AS #2
      OPEN f2$ FOR OUTPUT AS #3
'
' input the data
'
CLS

'    IF nc = 0 THEN
'
' Count number of response patterns
'
'    LOCATE 20, 1
'     PRINT "Number of response patterns counted:"
'     LINE INPUT #1, title$
'     nc = 0
'     IF EOF(1) GOTO 10
'5    LINE INPUT #1, junk$
'     IF INSTR(junk$, "1") > 0 OR INSTR(junk$, "2") > 0 THEN
'     nc = nc + 1
'     ELSE GOTO 10
'     END IF
'     LOCATE 1, 42
'     PRINT USING "#####"; nc;
'     GOTO 5
'10   CLOSE #1
'     OPEN f$ + ".tab" FOR INPUT AS #1
'     END IF

' run title
     
     LINE INPUT #1, title$
     'PRINT #2, title$
     'PRINT #2, ""
    
' Read in response patterns and observed and expected frequencies

     FOR I = 1 TO nc
         FOR j = 1 TO NR
            INPUT #1, IV(I, j)
         NEXT j
         INPUT #1, FO(I), FE(I)
     NEXT I
    
'Loop through pairs of raters

'     LOCATE 21, 1
      LOCATE 2, 1
      PRINT "Processing expected table for items: ";
     ' PRINT #2, "*** Expected data ***"
      FOR loop1 = 1 TO NR - 1
         LOCATE 2, 40
         PRINT loop1
         FOR loop2 = loop1 + 1 TO NR
            LOCATE 2, 45
            PRINT loop2

' Initialize marginal frequencies

            FOR I = 1 TO MC
            FOR j = 1 TO MC
               marge(I, j) = 0
               margo(I, j) = 0
            NEXT j
            NEXT I

' Loop through response patterns

            FOR K = 1 TO nc
            I = IV(K, loop1)
            j = IV(K, loop2)
               marge(I, j) = marge(I, j) + FE(K)
               margo(I, j) = margo(I, j) + FO(K)
            NEXT K

' Calculate G-squared statistic

       I = loop1
       j = loop2
       sum = 0!
       FOR ii = 1 TO 2
          FOR jj = 1 TO 2
           IF (marge(ii, jj) * margo(ii, jj) > 0) THEN
           sum = sum + 2 * margo(ii, jj) * LOG(margo(ii, jj) / marge(ii, jj))
           END IF
          NEXT jj
       NEXT ii
       g2(I, j) = sum

' Print expected two-way marginal tables

      'PRINT #2, ""
      'PRINT #2, "Marginal table for Items";
      'PRINT #2, USING "##"; loop1;
      'PRINT #2, " and";
      'PRINT #2, USING "##"; loop2;
      'PRINT #2, ""
      FOR I = 1 TO MC
         FOR j = 1 TO MC
            'PRINT #2, USING "  ##"; I; j;
            'PRINT #2, USING "  #####.##"; marge(I, j)
         NEXT j
      NEXT I
      a = marge(1, 1)
      b = marge(1, 2)
      c = marge(2, 1)
      d = marge(2, 2)

' adjust cells if there is a 0
' (if a row or column marginal is 0, that is a problem)

   IF (a * d = 0) THEN
      a = a + .5
      b = b - .5
      c = c - .5
      d = d + .5
   END IF
   IF (b * c = 0) THEN
      a = a - .5
      b = b + .5
      c = c + .5
      d = d - .5
   END IF

   psi = LOG((a * d) / (b * c))
   stderr = SQR(1 / a + 1 / b + 1 / c + 1 / d)
   'PRINT #2, ""
   'PRINT #2, USING " log odds ratio = #####.##"; psi
   'PRINT #2, USING " standard error = #####.###"; stderr

   psie(loop1, loop2) = psi
   sigma(loop1, loop2) = stderr

      NEXT loop2
      NEXT loop1
'
' now the observed data
'

'Loop through pairs of raters

'     LOCATE 22, 1
      PRINT "Processing observed table for items:";
      'PRINT #2, ""
      'PRINT #2, ""
      'PRINT #2, "*** Observed data ***"
      FOR loop1 = 1 TO NR - 1
         LOCATE 3, 40
         PRINT loop1
         FOR loop2 = loop1 + 1 TO NR
            LOCATE 3, 45
            PRINT loop2

' Initialize marginal frequencies

      FOR I = 1 TO MC
        FOR j = 1 TO MC
          margo(I, j) = 0
        NEXT j
      NEXT I

' Loop through cells

      FOR K = 1 TO nc
         I = IV(K, loop1)
         j = IV(K, loop2)
         margo(I, j) = margo(I, j) + FO(K)
      NEXT K

' Print observed two-way marginal tables

      'PRINT #2, ""
      'PRINT #2, "Marginal table for Items";
      'PRINT #2, USING "##"; loop1;
      'PRINT #2, " and";
      'PRINT #2, USING "##"; loop2;
      'PRINT #2, ""
      FOR I = 1 TO MC
         FOR j = 1 TO MC
            'PRINT #2, USING "  ##"; I; j;
            'PRINT #2, USING "  #####.#"; margo(I, j)
         NEXT j
      NEXT I
      a = margo(1, 1)
      b = margo(1, 2)
      c = margo(2, 1)
      d = margo(2, 2)

' adjust cells if there is a 0
' (if a row or column marginal is 0, that is a problem)

   IF (a * d = 0) THEN
      a = a + .5
      b = b - .5
      c = c - .5
      d = d + .5
   END IF
   IF (b * c = 0) THEN
      a = a - .5
      b = b + .5
      c = c + .5
      d = d - .5
   END IF

   psi = LOG((a * d) / (b * c))
   stderr = SQR(1 / a + 1 / b + 1 / c + 1 / d)
   'PRINT #2, ""
   'PRINT #2, USING " log odds ratio = #####.##"; psi
   'PRINT #2, USING " standard error = #####.###"; stderr

   psio(loop1, loop2) = psi

      NEXT loop2
      NEXT loop1
'
' calculate z score for each rater pair
'
      PRINT #3, "Conditional Dependence Diagnostics            "
      PRINT #3, title$
'PRINT #3, "                                              "
'PRINT #3, "------------------------------------------------------------------------"
'PRINT #3, " Items     Expected           95%      95%      Observed                "
'PRINT #3, "-------    log odds           lower    upper    log odds   z-           "
'PRINT #3, " i    j    ratio      s.e.    limit    limit    ratio      value     G2 "
'PRINT #3, "------------------------------------------------------------------------"

      PRINT #3, "------------------------------------------------------"
      PRINT #3, " Items     Expected           Observed                "
      PRINT #3, "-------    log odds           log odds   z-           "
      PRINT #3, " i    j    ratio      s.e.    ratio      value     G2 "
      PRINT #3, "------------------------------------------------------"

      sigflag = 0

'     LOCATE 23, 1
      PRINT "Calculating z and p values for items: ";
      FOR loop1 = 1 TO NR - 1
      LOCATE 4, 40
      PRINT loop1
      FOR loop2 = loop1 + 1 TO NR
      LOCATE 4, 45
      PRINT loop2
      I = loop1
      j = loop2

      z = (psio(I, j) - psie(I, j)) / sigma(I, j)

      PRINT #3, USING "##"; I;
      f$ = "   "
      IF (z <= -1.96 OR z >= 1.96) THEN f$ = " --"
      PRINT #3, f$;
      PRINT #3, USING "##"; j;
      PRINT #3, USING "######.##"; psie(I, j);
      PRINT #3, USING "######.###"; sigma(I, j);
'     PRINT #3, USING " #####.##"; psie(i, j) - 1.96 * sigma(i, j);
'     PRINT #3, USING " #####.##"; psie(i, j) + 1.96 * sigma(i, j);
      PRINT #3, USING "######.##"; psio(I, j);
      PRINT #3, USING "#######.##"; z;

      f$ = "  "
      IF (z <= -1.96 OR z >= 1.96) THEN
         f$ = " *"
         sigflag = 1
      END IF
      PRINT #3, f$;
      PRINT #3, USING "####.##"; g2(loop1, loop2)

      NEXT loop2
      NEXT loop1
      PRINT #3, "------------------------------------------------------"
'PRINT #3, "------------------------------------------------------------------------"
      IF sigflag = 1 THEN PRINT #3, "* p < .05, two-tailed."
      

      END

